from copy import deepcopy
import calendar
import datetime
import os
import sys

from obspy.core.event import read_events

from .lib_y2k import read_format, write_y2000_phase
from .liblog import getLogger
logger = getLogger()

from .. import get_format_dir
formats_dir = get_format_dir()

from .mappings import (hypoinv_weight_to_obspy_time_error,
                    obspy_time_error_to_hypoinv_weight)


def quakeml_to_y2k(quakemlfile):

    logger = getLogger()

    formats_dir = get_format_dir()

    y2000_format_file = os.path.join(formats_dir, 'format.Y2000_station_archive')
    y2000_header_file = os.path.join(formats_dir, 'format.Y2000_header_archive')

# 1. Read the Hypoinverse Y2000 archive format:
    y2k_format = read_format(y2000_format_file)
    hdr_format = read_format(y2000_header_file)

    event = None
    mag = None
    origin = None
    preferred_origin = False

    try:
        cat = read_events(quakemlfile, format="quakeml")
        event = cat[0]
        mag = event.preferred_magnitude() or event.magnitudes[0]
        origin = event.preferred_origin() or event.origins[0]

        if not origin:
            logger.warning("****** There is NO origin in this quakeml!")
    except:
        logger.error("Problem reading quakeml file=[%s]" % quakemlfile)
        raise

    y2k_origin = origin_to_y2k(origin, hdr_format)
    for magnitude in event.magnitudes:
        if magnitude.magnitude_type in ['M', 'ML', 'Ml', 'Mb', 'MS']:
            y2k_origin['ampmag'] = magnitude.mag
        #elif magnitude.magnitude_type in ['Mc', 'Md']:
        elif magnitude.magnitude_type in ['Mc']:
            y2k_origin['durmag'] = magnitude.mag
        elif magnitude.magnitude_type in ['Md']:
            y2k_origin['alt_dur_mag'] = magnitude.mag

    pref_magnitude = event.preferred_magnitude()
    if pref_magnitude:
        y2k_origin['prefmag_label'] = pref_magnitude.magnitude_type[-1]
        y2k_origin['prefmag'] = pref_magnitude.mag

    # MTH: just for testing
    #y2k_origin['event_id'] = 1234567890

    nS = 0
    nP = 0
    for arrival in origin.arrivals:
        if arrival.phase == 'P':
            nP += 1
        elif arrival.phase == 'S':
            nS += 1
        else:
            logger.info("Unkown arrival phase:%s" % arrival.phase)
    y2k_origin['number_S_times'] = nS

    # Note that we can also write out the origin line with this func:
    #write_y2000_phase(hdr_format, origin_to_y2k(origin, hdr_format))
    y2k_out = write_y2000_phase(hdr_format, y2k_origin)

    for arrival in origin.arrivals:
        y2k_phase = arrival_to_y2k(arrival, y2k_format)
        y2k_out += write_y2000_phase(y2k_format, y2k_phase)

    return y2k_out

def origin_to_y2k(origin, hdr_format):
    '''
    Convert a quakeml origin to y2k_origin dict

    Only reason for passing in hdr_format is so we can auto initialize
      all the y2k fields = None

    :param origin: origin to convert
    :type origin: obspy.core.event.origin.Origin

    :return: converted origin 
    :rtype: python dict

    '''

    y2k_origin = {}

    for k,v in hdr_format.items():
        y2k_origin[k] = None

    t = {}
    for x in {'year', 'month', 'day', 'hour', 'minute', 'second', 'microsecond'}:
        t[x] = getattr(origin.time, x)
    y2k_origin['year'] = t['year']
    y2k_origin['moddhhmi'] = "%02d%02d%02d%02d" % (t['month'], t['day'], t['hour'], t['minute'])
    y2k_origin['seconds'] = t['second'] + t['microsecond']/1e6

    lat = abs(origin.latitude)
    y2k_origin['lat_deg'] = int(lat)
    y2k_origin['lat_min'] = (lat - int(lat)) * 60.
    y2k_origin['n_or_s'] = '' if origin.latitude >= 0. else 'S'
    #print("lat=%f lat_deg=%d lat_min=%f" % (lat, y2k_origin['lat_deg'], y2k_origin['lat_min']))

    lon = abs(origin.longitude)
    y2k_origin['lon_deg'] = int(lon)
    y2k_origin['lon_min'] = (lon - int(lon)) * 60.
    y2k_origin['e_or_w'] = '' if origin.longitude <= 0. else 'E'

    y2k_origin['depth'] = origin.depth/1.e3
    if origin.depth_errors and origin.depth_errors.uncertainty:
        y2k_origin['error_vertical'] = origin.depth_errors.uncertainty/1.e3

    if origin.depth_type == "operator assigned":
        #program_remark       Auxiliary remark from program (i.e. "-" for depth fixed, etc
        y2k_origin['program_remark'] = '-'

    if origin.creation_info:
        if getattr(origin.creation_info, 'agency_id', None):
            y2k_origin['authority'] = origin.creation_info.agency_id[0]
        if getattr(origin.creation_info, 'version', None):
            y2k_origin['version_info'] = origin.creation_info.version

    #number_S_times          Number of S times with weights greater than 0.1.
    #number_P_first_motions  Number of P first motions. *
    if origin.quality:
        if getattr(origin.quality, 'azimuthal_gap', None):
            y2k_origin['azgap'] = origin.quality.azimuthal_gap
        if getattr(origin.quality, 'used_phase_count', None):
            y2k_origin['n_P_and_S_times'] = origin.quality.used_phase_count
        if getattr(origin.quality, 'associated_phase_count', None):
            y2k_origin['n_valid_P_and_S_reads'] = origin.quality.associated_phase_count
        if getattr(origin.quality, 'standard_error', None):
            y2k_origin['rms'] = origin.quality.standard_error
        if getattr(origin.quality, 'minimum_distance', None):
            y2k_origin['min_dist'] = origin.quality.minimum_distance * 111.19

    if origin.origin_uncertainty:
        uncertainty = origin.origin_uncertainty
        if getattr(uncertainty, 'confidence_ellipsoid', None):
            y2k_origin['pri_error_az']   = uncertainty.confidence_ellipsoid.major_axis_azimuth
            y2k_origin['pri_error_dip']  = uncertainty.confidence_ellipsoid.major_axis_plunge
            y2k_origin['pri_error_size'] = uncertainty.confidence_ellipsoid.semi_major_axis_length/1.e3
            y2k_origin['sm_error_size']  = uncertainty.confidence_ellipsoid.semi_minor_axis_length/1.e3
            y2k_origin['int_error_size'] = uncertainty.confidence_ellipsoid.semi_intermediate_axis_length/1.e3

            #These don't exist
            #y2k_origin['int_error_az']   = uncertainty.confidence_ellipsoid.major_axis_azimuth
            #y2k_origin['int_error_dip']  = uncertainty.confidence_ellipsoid.major_axis_plunge
            #y2k_origin['int_error_size'] = uncertainty.confidence_ellipsoid.semi_minor_axis_length/1.e3
        else:
            if getattr(uncertainty, 'max_horizontal_uncertainty', None):
                y2k_origin['pri_error_size']  = uncertainty.max_horizontal_uncertainty /1.e3
            if getattr(uncertainty, 'azimuth_max_horizontal_uncertainty', None):
                y2k_origin['pri_error_az']  = uncertainty.azimuth_max_horizontal_uncertainty
            if getattr(uncertainty, 'min_horizontal_uncertainty', None):
                y2k_origin['sm_error_size']  = uncertainty.min_horizontal_uncertainty /1.e3 

        if getattr(uncertainty, 'horizontal_uncertainty', None):
            y2k_origin['error_horizontal'] = uncertainty.horizontal_uncertainty/1.e3

    if origin.creation_info:
        if getattr(origin.creation_info, 'agency_id', None):
            y2k_origin['authority'] = origin.creation_info.agency_id[0]
        if getattr(origin.creation_info, 'version', None):
            y2k_origin['version_info'] = origin.creation_info.version

    return y2k_origin

def arrival_to_y2k(arrival, y2k_format):
    '''
    Convert a quakeml arrival to y2k arrival dict

    Only reason for passing in y2k_format is so we can auto initialize
      all the y2k_phase fields = None

    :param origin: origin to convert
    :type origin: obspy.core.event.origin.Origin

    :return: converted origin 
    :rtype: python dict
    '''


    y2k_phase = {}

    for k,v in y2k_format.items():
        y2k_phase[k] = None

    pick = arrival.pick_id.get_referred_object()
    y2k_phase['sta']  = pick.waveform_id.station_code
    y2k_phase['net']  = pick.waveform_id.network_code
    y2k_phase['chan'] = pick.waveform_id.channel_code
    y2k_phase['loc']  = pick.waveform_id.location_code

    t = {}
    for x in {'year', 'month', 'day', 'hour', 'minute', 'second', 'microsecond'}:
        t[x] = getattr(pick.time, x)
    y2k_phase['year'] = t['year']
    y2k_phase['moddhhmi'] = "%02d%02d%02d%02d" % (t['month'], t['day'], t['hour'], t['minute'])

    phase = 'P'

    if arrival.phase[0] == 'S':
        phase = 'S'
    onset = " "
    if pick.onset == 'impulsive':
        onset = 'I'
    rmk = '%srmk' % (phase)
    y2k_phase[rmk] = "%1s%1s" % (onset, phase)

    if phase == 'P' and hasattr(pick, 'polarity'):
        if pick.polarity == 'positive':
            y2k_phase['PUpDown'] = 'U'
        elif pick.polarity == 'negative':
            y2k_phase['PUpDown'] = 'D'

    y2k_phase['%ssec' % phase]    = t['second'] + t['microsecond']/1e6
    y2k_phase['%srms' % phase]    = arrival.time_residual
    y2k_phase['%swtUsed' % phase] = arrival.time_weight

    # MTH: Does this need to be flagged phase='S' so that wt=0 --> wt=5 for hypoinv, etc ?
    y2k_phase['%swtCode' % phase] =  obspy_time_error_to_hypoinv_weight(pick.time_errors.uncertainty)

    if arrival.azimuth:
        y2k_phase['Azim']    = "%.2f" % arrival.azimuth
    if arrival.distance:
        y2k_phase['Dist']    = arrival.distance * 111.19 #y2k dist = km vs. obspy dist = deg
    if arrival.takeoff_angle:
        y2k_phase['Angle']   = arrival.takeoff_angle

    return y2k_phase




import os

import logging

from .lib.qml_to_y2k import quakeml_to_y2k
from .lib.liblog import getLogger, processCmdLine

def main():

    logger = getLogger()
    quakemlfile = processCmdLine('quakeml')

    infile = os.path.basename(quakemlfile)
    outfile = infile.replace('.qml', '.y2k')
    if outfile == infile:
        outfile = infile.replace('.xml', '.y2k')
    if outfile == infile:
        logger.info("Infile doesn't have expected suffix (.qml, .xml) --> Add .y2k to infile")
        outfile += ".y2k"

    logger.info("infile=%s --> outfile=./%s" % (quakemlfile, outfile))

    y2k = quakeml_to_y2k(quakemlfile)

    with open(outfile, 'w') as fh:
        fh.write(y2k)

    return

if __name__ == "__main__":
    main()



from copy import deepcopy
import calendar
import datetime
import os
import sys

from obspy.core.event.base import ConfidenceEllipsoid
from obspy.core.event.base import CreationInfo
from obspy.core.event.base import QuantityError
from obspy.core.event.base import WaveformStreamID
from obspy.core.event.magnitude import Amplitude
from obspy.core.event.origin import Arrival
from obspy.core.event.origin import Origin
from obspy.core.event.origin import OriginQuality, OriginUncertainty
from obspy.core.event.origin import Pick
from obspy.core.utcdatetime import UTCDateTime

from obspy.core.event import Catalog
from obspy.core.event import read_events
from obspy.core.event.event import Event
from obspy.core.event.magnitude import Magnitude

from .mappings import (hypoinv_weight_to_obspy_time_error,
                    obspy_time_error_to_hypoinv_weight)

from .lib_y2k import read_format, read_arc_shadow_file
from .liblog import getLogger
logger = getLogger()

from .. import get_format_dir
formats_dir = get_format_dir()

def y2k_to_quakeml(arc_file):

    fname = 'y2k-to-quakeml'

# 1. Read the Hypoinverse Y2000 archive format:
    y2000_format_file = os.path.join(formats_dir, 'format.Y2000_station_archive')
    y2000_header_file = os.path.join(formats_dir, 'format.Y2000_header_archive')
    y2k_format = read_format(y2000_format_file)
    hdr_format = read_format(y2000_header_file)

    # Read in the y2k arc file
    logger.debug("Read y2k arc file into origin + y2k_picks")
    (y2k, y2k_origin)  = read_arc_shadow_file(arc_file, y2k_format, hdr_format)
    if y2k_origin is None:
        logger.error("%s: Problem reading arc_file=[%s] --> Exitting" % (fname, arc_file))
        exit(2)

    # Set up epoch times (may be corrected by leap seconds in the future)
    yyyymoddhhmi = str(y2k_origin['year']) + y2k_origin['moddhhmi']
    moddhhmi = y2k_origin['moddhhmi']
    try:
        (month, day, hour, min) = (int(moddhhmi[0:2]), int(moddhhmi[2:4]), int(moddhhmi[4:6]), int(moddhhmi[6:8]))
        year = int(y2k_origin['year'])
        isecs= int(y2k_origin['seconds'])
        xsecs= y2k_origin['seconds'] - isecs
    except ValueError:
        raise
    da = datetime.datetime(year, month, day, hour, min, isecs)
    # epoch  = origin timestamp   
    # epoch0 = origin timestamp without seconds = base for amps + codas
    epoch = calendar.timegm(da.timetuple()) + xsecs
    epoch0 = calendar.timegm(datetime.datetime(year, month, day, hour, min, 0).timetuple())
    # Do we need to shift times by leap seconds ?  # e.g, are we using a database ?
    use_db = 0
    if use_db: # MTH: this code is implemented in aqms-pdl and depends on db connection:
        epoch += get_leap_seconds(y2k_origin, event)
        epoch0 += get_leap_seconds(y2k_origin, event)

    y2k_origin['datetime'] = epoch

    event = Event()
    # Convert y2k origin to quakeml origin
    origin = y2k_to_origin(y2k_origin)
    event.preferred_origin_id = origin.resource_id

    nsta_ml = y2k_origin['tot_amp_mag_weights']
    nobs_ml = 2*nsta_ml  # Because each station has two amp observations
    nsta_mc = y2k_origin['tot_dur_mag_weights']
    nobs_mc = nsta_mc
    nsta_my = y2k_origin['tot_alt_dur_mag_weights']
    nobs_my = nsta_my

    found_dict = {'ml_found':0,
                  'mc_found':0,
                  'my_found':0,
                 }

    # This mapping comes from UUtah and may not be general!
    #              y2k field     mag_type   nsta    found_flag
    y2k_mag_dict = {'ampmag' :      ('ML', nsta_ml, 'ml_found'),
                    'durmag' :      ('Mc', nsta_mc, 'mc_found'),
                    'alt_dur_mag' : ('Md', nsta_my, 'my_found'),
                   }
    magnitudes = []
    for y2k_mag_key in y2k_mag_dict.keys():
        if y2k_origin[y2k_mag_key] > -9:
            logger.info("%s: Found ampmag:%.2f" % (fname, y2k_origin[y2k_mag_key]))
            #ml_found = 1
            (mag_type, nsta, found_key) = y2k_mag_dict[y2k_mag_key]
            found_dict[found_key] = 1
            magnitude = Magnitude()
            magnitude.mag = y2k_origin[y2k_mag_key]
            magnitude.magnitude_type = mag_type
            magnitude.origin_id = origin.resource_id
            magnitude.station_count = nsta
            magnitudes.append(magnitude)

    if magnitudes:
        event.magnitudes = magnitudes
        event.preferred_magnitude_id = magnitudes[0].resource_id

    nmags_found = 0
    for k in found_dict.keys():
        nmags_found += found_dict[k]
    #nmags_found = ml_found + mc_found + my_found

    logger.info("%s: Hypoinv file contains n=[%d] valid mags:" % (fname, nmags_found))
    logger.info("%s: ml_found=%d mc_found=%d my_found=%d" % (fname, found_dict['ml_found'], 
                                                             found_dict['mc_found'], found_dict['my_found']))
    logger.info("%s: Hypoinv prefmag_label=[%s] prefmag=%3.2f" % (fname, y2k_origin['prefmag_label'], y2k_origin['prefmag']))

    if found_dict['ml_found'] and y2k_origin['prefmag_label'] != 'L':
        logger.warning("%s: ml_found but prefmag_label=[%s] and *not* 'L' --> making ML preferred anyway!" %
                    (fname, y2k_origin['prefmag_label']))
    if found_dict['ml_found'] and y2k_origin['prefmag'] != y2k_origin['ampmag']:
        logger.warning("%s: ml_found but prefmag=[%3.2f] != ampmag=[%3.2f]" %
                    (fname, y2k_origin['prefmag'], y2k_origin['ampmag']))

    # Should check this against y2k_origin['prefmag_label'] and y2k_origin['prefmag']
    if found_dict['ml_found']:
        y2k_origin['prefmagType'] = 'l'
    elif found_dict['mc_found']:
        y2k_origin['prefmagType'] = 'd'

# 5. Now count # of arrivals, amps, codas - and use to get sequences
    #write_y2000_phase(hdr_format, y2k_origin)

    codas = []
    amps  = []
    Parrs = []
    Sarrs = []
    all_arrivals = []

    avgMag = {}
    flag119 = {}

    auth = y2k_origin['authority']
# 6. Each y2k arrival line must be sorted for injection into one or more of the tables: {arrival, amp, coda}
    #for sta in y2k:
        #for chan in y2k[sta]:
            #arrival = y2k[sta][chan]
    if 1:
        for arrival in y2k:
            #arrival = y2k[sta][chan]
            #arrival['auth'] = 'UU'
            arrival['auth'] = auth
            # arrival['orid'] = orid
            # arrival['subsource'] = 'uping'
            logger.debug("%s: Process arrival:%s" % (fname, arrival['string']))
            arrival_datetime = str(arrival['year']) + arrival['moddhhmi']

            validP = 0
            validS = 0
            validAmp = 0
            validCoda = 0

            # Is P arrival valid
            if arrival['PwtCode'] >= 0 and arrival['Psec'] > 0.:
                validP = 1
            elif arrival['PwtCode'] >= 0 and arrival['Psec'] == 0. and arrival_datetime != yyyymoddhhmi:
                validP = 1
            if validP:
                arrival['iphase']='P'
                arrival['datetime']=epoch0 + arrival['Psec']
                Parrs.append(deepcopy(arrival))
                all_arrivals.append(deepcopy(arrival))
                logger.debug("%s: Found P arrival: %s %s %s %5.2f" % (fname, arrival['sta'], arrival['chan'],
                              arrival_datetime, arrival['Psec']))

            # Is S arrival valid
            if arrival['SwtCode'] >= 0 and arrival['Ssec'] > 0. and not arrival['Ssec'] == 60.00:
                validS = 1
            elif arrival['SwtCode'] >= 0 and arrival_datetime != yyyymoddhhmi:
                if not validP: #If no P info, then I have to assume the S time is correct (Ssec exactly equals 0 or 60):
                    validS = 1
                elif arrival['Ssec'] > arrival['Psec']: #If we have P, then check S > P in relative sense:
                    validS = 1 
            if validS:
                arrival['iphase']='S'
                arrival['datetime']=epoch0 + arrival['Ssec']
                Sarrs.append(deepcopy(arrival))
                all_arrivals.append(deepcopy(arrival))
                logger.debug("%s: Found S arrival: %s %s %s %5.2f" % (fname, arrival['sta'], arrival['chan'], \
                              arrival_datetime, arrival['Ssec']))

            # Is Amp valid
            if arrival['Amp'] > 0:
                if arrival['AmpMag'] == 0.:
                    logger.error("%s: sta=%s chan=%s: has AmpMag == 0. --> Skip" %
                                 (fname, arrival['sta'], arrival['chan'], arrival['coda']))
                else:
                    validAmp = 1
                    if arrival['chan'][2] == 'H':
                        # Use this chan to set AmpMag on the ?HN, ?HE chans
                        #print '  This is a ficticious H comp --> get AmpMag=[%3.2f]' % arrival['AmpMag']
                        logger.debug("%s: Found Amp ?HH chan: %s %s %s %5.2f" % (fname, arrival['sta'], arrival['chan'], \
                                      arrival_datetime, arrival['Amp']))
                        avgMag[sta]  = arrival['AmpMag']
                        flag119[sta] = arrival['blank4']
                        if arrival['blank4'] == 'X':
                            logger.debug("%s: Found ?HH chan: %s %s %s %5.2f with col.199='X' " % (fname, arrival['sta'], arrival['chan'], \
                                          arrival_datetime, arrival['Amp']))
                    else:
                        #arrival['datetime']=0.
                        arrival['datetime']=y2k_origin['datetime']
                        # This is from U Utah:
                        # magcorr = magcorr_lookup(arrival, epoch0, station_mag_corrs)
                        # For now do this:
                        magcorr = 0.0
                        arrival['magcorr']=magcorr

                        amps.append(deepcopy(arrival))
                        logger.debug("%s: Found Amp arrival: %s %s %s %5.2f magcorr:%.2f" % (fname, arrival['sta'], arrival['chan'], \
                                       arrival_datetime, arrival['Amp'], arrival['magcorr']))

            if 'coda' in arrival:
                if arrival['DurMag'] <= 0.:
                    logger.error("%s: sta=%s chan=%s: Has coda shadow line=[%s] but missing DurMag! --> Skip" %
                                 (fname, arrival['sta'], arrival['chan'], arrival['coda']))
                else:
                    validCoda = 1
                    arrival['datetime']=epoch0
                    arrival['tau']=float(arrival['coda']['end'] - arrival['coda']['beg'])
                    arrival['afree']=arrival['coda']['logA0']
                    arrival['qfree']=arrival['coda']['alpha']
                    arrival['beg']=arrival['coda']['beg']
                    arrival['end']=arrival['coda']['end']
                    codas.append(deepcopy(arrival))
                    logger.debug("%s: Found Coda arrival: %s %s %s %5.2f" % (fname, arrival['sta'], arrival['chan'],
                                    arrival_datetime, arrival['DurMag']))

            if not validP and not validS and not validAmp and not validCoda:
                logger.error("%s: Unable to scan P,S,Amp,Coda info from arrival !!" % (fname))

# For stations with ?HH channel amp, assign the corresponding AmpMag to the ?HN & ?HE channels:
    for amp in amps:
        if amp['sta'] in avgMag.keys():
                amp['AmpMag'] = avgMag[amp['sta']]
        else:
                logger.warning("%s: sta=[%s] NOT in avgMag keys" % (fname, amp['sta']))

# We want to set ?HN,?HE amp mag weight based on whether there's an 'X' or not in col. 119 of the ?HH hyp2000 shadow line 
        if amp['sta'] in flag119.keys():
                amp['col_119'] = flag119[amp['sta']]
        else:
                logger.warning("%s: sta=[%s] NOT in flag119 keys" % (fname, amp['sta']))


    narrivals = len(Parrs) + len(Sarrs)
    logger.info("%s: Got %d P-arrivals + %d S-arrivals" % (fname, len(Parrs), len(Sarrs)))

    arrivals = []
    picks = []

    #for arr in Parrs + Sarrs:
    for arr in all_arrivals:
        arrival, pick = y2k_phase_to_arrival(arr, arr['iphase'])
        arrivals.append(arrival)
        picks.append(pick)

    logger.info("%s: Scanned [%d] P/S arrivals" % (fname, narrivals))
    for arr in arrivals:
        pick = arr.pick_id.get_referred_object()
        wid = pick.waveform_id
        if wid.location_code:
            wave_id = "%2s.%4s.%2s-%3s" % (wid.network_code, wid.station_code, wid.location_code, wid.channel_code)
        else:
            wave_id = "%2s.%4s.%3s" % (wid.network_code, wid.station_code, wid.channel_code)
        logger.info("arr: %s [%s] %f" % (wave_id, arr.phase, pick.time))

    amplitudes = []
    logger.info("%s: Scanned [%d] amplitudes" % (fname, len(amps)))
    for amp in amps:
        amplitude = y2k_phase_to_amplitude(amp)
        amplitudes.append(amplitude)
        #print(amplitude)
        wid = amplitude.waveform_id
        if wid.location_code:
            wave_id = "%2s.%4s.%2s-%3s" % (wid.network_code, wid.station_code, wid.location_code, wid.channel_code)
        else:
            wave_id = "%2s.%4s.%3s" % (wid.network_code, wid.station_code, wid.channel_code)
        logger.info("amp: %s [%s] %f" % (wave_id, amplitude.type, amplitude.generic_amplitude))

    logger.info("%s: Scanned [%d] codas" % (fname, len(codas)))
    # These seem to be coda (per channel) magnitudes
    for arrival in codas:
        logger.info("%s: Coda: %s %s %s %5.2f" % (fname, arrival['sta'], arrival['chan'],
                      arrival_datetime, arrival['DurMag']))

    origin.arrivals = arrivals
    event.origins = [origin]
    event.picks = picks
    event.amplitudes = amplitudes

    if y2k_origin['prefmag_label']:
        logger.info("Found prefmag:%s val:%.2f" % (y2k_origin['prefmag_label'], y2k_origin['prefmag']))

    catalog = Catalog(events=[event])

    return catalog

def y2k_to_origin(y2k_origin):
    '''
    Convert a y2k_origin to quakeml Origin

    :param y2k_origin: origin to convert
    :type origin: y2k origin dict

    :return: converted origin 
    :rtype: obspy.core.event.origin.Origin
    '''

    origin = Origin()

    year = y2k_origin['year']
    mo = y2k_origin['moddhhmi'][0:2]
    dd = y2k_origin['moddhhmi'][2:4]
    hh = y2k_origin['moddhhmi'][4:6]
    mi = y2k_origin['moddhhmi'][6:8]
    ss = y2k_origin['seconds']
    date = "%s-%s-%sT%s:%s:%s" % (year, mo, dd, hh, mi, ss)
    origin.time = UTCDateTime(date)

    longitude = y2k_origin['lon_deg'] + y2k_origin['lon_min']/60.0
    if y2k_origin['e_or_w'] != 'E':
        longitude *= -1

    latitude = y2k_origin['lat_deg'] + y2k_origin['lat_min']/60.0
    if y2k_origin['n_or_s'] == 'S':
        longitude *= -1

    origin.longitude = longitude
    origin.latitude = latitude
    origin.depth = y2k_origin['depth'] * 1.e3  # y2k depth [km] --> obspy depth [m]
    qe = QuantityError()
    qe.uncertainty = y2k_origin['error_vertical'] * 1.e3  # y2k depth [km] --> obspy depth [m]
    # qe.lower_uncertainty
    # qe.upper_uncertainty
    # qe.confidence_level
    origin.depth_errors = qe

    if y2k_origin['program_remark'] == " -":
        origin.depth_type = "operator assigned"
    origin.region = y2k_origin['region']

    quality = OriginQuality()
    #quality.associated_phase_count
    #quality.used_phase_count
    quality.standard_error = y2k_origin['rms']
    quality.azimuthal_gap = y2k_origin['azgap']
    quality.minimum_distance = y2k_origin['min_dist'] / 111.19
    quality.used_phase_count = y2k_origin['n_P_and_S_times']
    quality.associated_phase_count = y2k_origin['n_valid_P_and_S_reads']
    origin.quality = quality

    uncertainty = OriginUncertainty()
    uncertainty.horizontal_uncertainty = y2k_origin['error_horizontal'] * 1.e3 # km --> m
    uncertainty.max_horizontal_uncertainty = y2k_origin['pri_error_size'] * 1.e3
    uncertainty.azimuth_max_horizontal_uncertainty = y2k_origin['pri_error_az']
    uncertainty.min_horizontal_uncertainty = y2k_origin['sm_error_size'] * 1.e3
    origin.origin_uncertainty = uncertainty

    confidence_ellipsoid = ConfidenceEllipsoid()
    uncertainty.confidence_ellipsoid = confidence_ellipsoid
    confidence_ellipsoid.major_axis_azimuth = y2k_origin['pri_error_az']
    confidence_ellipsoid.major_axis_plunge  = y2k_origin['pri_error_dip']
    confidence_ellipsoid.semi_major_axis_length = y2k_origin['pri_error_size'] * 1.e3
    confidence_ellipsoid.semi_minor_axis_length = y2k_origin['sm_error_size'] * 1.e3
    confidence_ellipsoid.semi_intermediate_axis_length = y2k_origin['int_error_size'] * 1.e3

    creation_info = CreationInfo()
    creation_info.agency_id = y2k_origin['authority']
    creation_info.version = y2k_origin['version_info']
    origin.creation_info = creation_info

    return origin


def y2k_phase_to_arrival(y2k_phase, phase):
    '''
    Convert a y2k phase dict to obspy arrival

    :param phase: 'P' or 'S'
    :type phase: str
    :param y2k_phase: y2k_phase
    :type y2k_phase: python dict

    :return: converted phase 
    :rtype: obspy arrival
    '''

    arrival = Arrival()
    arrival.phase = phase
    if y2k_phase['Azim'] > 0:
        arrival.azimuth = y2k_phase['Azim']
    # epicentral dist in deg
    if y2k_phase['Dist'] > 0:
        arrival.distance = y2k_phase['Dist'] / 111.19  #Convert y2k dist [km] to obspy dist [deg]
    # takeoff angle (deg) measured from downward vertical
    if y2k_phase['Angle'] > 0:
        arrival.takeoff_angle = y2k_phase['Angle']    ###   Verify y2k uses same takeoff angle convention (?)
    arrival.time_residual = y2k_phase['%srms' % phase]
    arrival.time_weight = y2k_phase['%swtUsed' % phase]

    waveform_id = WaveformStreamID(y2k_phase['net'], y2k_phase['sta'])
    waveform_id.channel_code = y2k_phase['chan']
    if y2k_phase['loc'] != -9:
        waveform_id.location_code = y2k_phase['loc']

    pick = Pick()
    pick.waveform_id = waveform_id
    year = y2k_phase['year']
    mo = y2k_phase['moddhhmi'][0:2]
    dd = y2k_phase['moddhhmi'][2:4]
    hh = int(y2k_phase['moddhhmi'][4:6])
    mi = int(y2k_phase['moddhhmi'][6:8])
    ss = float(y2k_phase['%ssec' % phase])
    if ss > 60.:
        mins = int(ss // 60.)
        ss = ss % (60. * mins)
        mi += mins

    date = "%s-%s-%sT%s:%s:%s" % (year, mo, dd, hh, mi, ss)
    pick.time = UTCDateTime(date)
    #pick.backazimuth = 

    arrival.pick_id = pick.resource_id

    onset = y2k_phase['%srmk' % phase][0]

    if onset == 'I':
        pick.onset = "impulsive"
    elif onset == 'E':
        pick.onset = "emergent"
    pick.phase_hint = phase

    if phase == 'P':
        if y2k_phase['PUpDown'] == 'U':
            pick.polarity = 'positive'
        elif y2k_phase['PUpDown'] == 'D':
            pick.polarity = 'negative'

    time_errors = QuantityError() 
    time_errors.uncertainty = hypoinv_weight_to_obspy_time_error(y2k_phase['%swtCode' % phase])
    pick.time_errors = time_errors

    return arrival, pick

'''
OrderedDict([('sta', 'YNR'), 
('string', 'YNR  WY  HHE    4201112131506    0-999  0    0   0   00.21966 0  0   0   0 542 6900  0       9-9.174   0   0  L01 1   X '),
('net', 'WY'), ('comp1', ''), ('chan', 'HHE'), ('Prmk', '  '), ('PUpDown', ''), ('PwtCode', 4), 
('year', 2011), ('moddhhmi', '12131506'), ('Psec', 0.0), ('Prms', -9.99), ('PwtUsed', 0.0), 
('Ssec', 0.0), ('Srmk', '  '), ('SwtCode', 0), ('Srms', 0.0), 
('Amp', 0.21966), ('AmpCode', 0), ('SwtUsed', 0.0), 
('Pdelay', 0.0), ('Sdelay', 0.0), ('Dist', 54.2), ('Angle', 69.0), ('AmpMagC', 0), ('DurMagC', 0), 
('PerMax', 0.0), ('StaRmk', ''), ('CodaDur', -9), ('Azim', 9.0), ('DurMag', -9.0), ('AmpMag', 2.1), 
('Pimport', 0.0), ('Simport', 0.0), ('DataSource', ''), ('DurCode', ''), 
('AmpLabel', 'L'), ('loc', '01'), ('AmpType', 1), ('comp3', ''), ('blank4', 'X'), ('blank5', ''), 
('auth', 'UU'), ('subsource', 'uping'), ('datetime', 1323788816.88), ('magcorr', 0.0), ('col_119', '')])
'''

def y2k_phase_to_amplitude(y2k_phase):
    '''
    Convert a y2k phase dict to obspy amplitude

    :param phase: 'P' or 'S'
    :type phase: str
    :param y2k_phase: y2k_phase
    :type y2k_phase: python dict

    :return: converted phase 
    :rtype: obspy amplitude
    '''

    waveform_id = WaveformStreamID(y2k_phase['net'], y2k_phase['sta'])
    waveform_id.channel_code = y2k_phase['chan']
    if y2k_phase['loc'] != -9:
        waveform_id.location_code = y2k_phase['loc']

    amplitude = Amplitude()
    amplitude.waveform_id = waveform_id
    amplitude.generic_amplitude = y2k_phase['Amp']
    amplitude.type = 'A'                # unspecified amplitude reading 
    if y2k_phase['AmpLabel'] == 'L':
        amplitude.type = 'AML'          # amplitude reading for local magnitude

    amplitude.scaling_time = y2k_phase['datetime']   # Check this

    #amplitude.unit = 
    #amplitude.per = 
    #amplitude.snr = 
    #amplitude.time_window = 
    #amplitude.pick_id = 

    '''
    pick = Pick()
    pick.waveform_id = waveform_id
    year = y2k_phase['year']
    mo = y2k_phase['moddhhmi'][0:2]
    dd = y2k_phase['moddhhmi'][2:4]
    hh = int(y2k_phase['moddhhmi'][4:6])
    mi = int(y2k_phase['moddhhmi'][6:8])
    ss = float(y2k_phase['%ssec' % phase])
    if ss > 60.:
    '''
    return amplitude



import os

from .lib.liblog import getLogger, processCmdLine
from .lib.nordic_to_qml import seisan_to_quakeml

def main():

    logger = getLogger()

    seisan_file, args = processCmdLine('infile')

    infile = os.path.basename(seisan_file)
    # What do seisan files end with ???????
    outfile = infile.replace('.y2k','.qml')
    if outfile == infile:
        logger.info("Infile doesn't have expected .y2k suffix --> Add .qml to infile")
        outfile += ".qml"

    logger.info("infile=%s --> outfile=./%s" % (seisan_file, outfile))

    cat = seisan_to_quakeml(seisan_file, args)
    cat.write(outfile, format="QUAKEML")

    return

if __name__ == "__main__":
    main()



from .liblog import getLogger
logger = getLogger()

'''
  Hypoinverse/Nordic-type4    aqms-pdl ObsPy map           AQMS db
    Weighting-indicator     obspy.pick.time_error       Arrival.quality
        P              S
    ==============   =====    ============              ===========
        0 or blank     5        <= 0.06                     1.0         Best
        1              6        <= 0.12                     0.75
        2              7        <= 0.30                     0.50
        3              8        <= 0.60                     0.25
        4              9         > 0.60                     0.10        Worst
'''
weight_to_error = {
    0: 0.06,
    1: 0.12,
    2: 0.30,
    3: 0.60,
    4: 0.65,
}


def hypoinv_weight_to_obspy_time_error(weight):

    fname = 'hypoinv_weight_to_obspy_time_error'
                                                 # Seisan blank or 0 = full weight
    if (isinstance(weight, str) and len(weight.strip()) == 0) or \
       (isinstance(weight, int) and weight == -9): # but y2k reader sets empty fields = -9
        weight = 0

    try:
        wt = int(weight)
    except:
        logger.error("Can't convert int weight=[%s] to time_error!" % weight)
        raise

    if wt > 9:
        logger.error("weight=[%d] > 9 --> mapping only goes up to 9!" % wt)
        exit(2)

    if wt > 4:
        wt = wt - 5

    logger.info("%s: wt:%d --> time_error:%.2f" % (fname, weight, weight_to_error[wt]))
    #print("%s: wt:%d --> time_error:%.2f" % (fname, weight, weight_to_error[wt]))

    return weight_to_error[wt]


def obspy_time_error_to_hypoinv_weight(time_error):

    fname = 'obspy_time_error_to_hypoinv_weight'

    try:
        x = float(time_error)
    except:
        logger.error("Can't convert time_error=[%s] to float!" % time_error)
        raise

    # dict keys are only sorted for python >= 3.7
    for wt in sorted(weight_to_error.keys()):
        if float(time_error) <= weight_to_error[wt]:
            #print("time_error:%f <= wt_to_error[%d]=%f" %
                  #(time_error, wt, weight_to_error[wt]))
            return wt

    return sorted(weight_to_error.keys())[-1]




import logging
import os
import sys

DEFAULT_LOG_LEVEL = 'INFO'

# logging.getLogger() will return the root logger of the parent calling func
#   if it wasn't configured (no handlers) then we'll add a default console handler
def getLogger():
    logger = logging.getLogger()
    if len(logger.handlers) == 0:
        ch = logging.StreamHandler()
        formatter = logging.Formatter('[%(levelname)7s] %(message)s')
        ch.setFormatter(formatter)
        logger.addHandler(ch)
        #logger.setLevel(logging.DEBUG)
        logger.setLevel(logging.getLevelName(DEFAULT_LOG_LEVEL))
    return logger



import argparse
def processCmdLine(fname):
    '''
    Pass in fname to build argparse option: >cmd.py --fname=....
       and return the path in fname
    '''

    logger = getLogger()

    parser = argparse.ArgumentParser()
    optional = parser._action_groups.pop()
    required = parser.add_argument_group("required arguments")

    #required.add_argument("--y2kfile", type=str, metavar='Path to y2k arc file', required=True)
    file_arg = "--" + fname
    required.add_argument(file_arg, type=str, metavar='Path to input file', required=True)
    optional.add_argument("--loglevel", type=str, metavar='LOGLEVEL: INFO, WARN, etc')
    optional.add_argument("--network-code", type=str, metavar='Hard-code network code in output quakeml file')
    optional.add_argument("--fix-amptype", action='store_true')
    args, unknown = parser.parse_known_args()

    usage = "Usage: %s --loglevel=WARN --network-code TX --fix-amptype" % os.path.basename(sys.argv[0])

    if unknown:
        for k in unknown:
            print("Unknown arg:%s" % k)
        print(usage)
        exit(2)


    loglevels = {"ERROR": logging.ERROR,
                 "WARN": logging.WARNING,
                 "WARNING": logging.WARNING,
                 "INFO": logging.INFO,
                 "DEBUG": logging.DEBUG,
                 "CRITICAL": logging.CRITICAL,
                }
    msg = " ".join(list(loglevels.keys()))

    loglevel = None
    if args.loglevel:
        if args.loglevel in loglevels:
            loglevel = loglevels[args.loglevel]
        else:
            logger.warning("Unknown --loglevel=%s (not in:{%s}) --> Use default loglevel=%s" % 
                        (args.loglevel, msg, DEFAULT_LOG_LEVEL))
            loglevel = None

    if loglevel:
        logger.setLevel(loglevel)

    return getattr(args, fname), args


if __name__ == "__main__":
    main()





import os

from obspy.core.event.origin import Arrival
from obspy.core.event.origin import Origin
from obspy.core.event.origin import OriginQuality, OriginUncertainty
from obspy.core.event.origin import Pick
from obspy.core.event.base import CreationInfo
from obspy.core.event.base import QuantityError
from obspy.core.event.base import WaveformStreamID
from obspy.core.utcdatetime import UTCDateTime
from obspy.core.event.magnitude import Magnitude
from obspy.core.event.magnitude import Amplitude

from .mappings import (hypoinv_weight_to_obspy_time_error,
                    obspy_time_error_to_hypoinv_weight)

FIELD_UNSET = -9 # This should likely be changed in parse_y2k_line to something more like NaN 

from obspy.core.event import Catalog
from obspy.core.event.event import Event

from .liblog import getLogger
logger = getLogger()

from .lib_y2k import read_format, parse_y2k_line

from .. import get_format_dir
formats_dir = get_format_dir()


def seisan_to_quakeml(seisan_file, args=None):

    '''
    Note: Type 1 line must be the first, all type 4 lines should be together and the last line must be blank
          Type E Line (Optional): Hyp error estimates
          Type H line, High accuracy hypocenter line
    '''

    formats = {}
    #nordic_types = ['1', '4', '6', '7', 'E', 'H', 'I']
    nordic_types = ['1', '4', 'E', 'H']

    logger.info("formats_dir=%s" % formats_dir)

    for ntype in nordic_types:
        #format_file = '../formats/format.nordic.type%s' % ntype
        format_file = os.path.join(formats_dir, 'format.nordic.type%s' % ntype)
        formats[ntype]= read_format(format_file)

    with open(seisan_file, 'r') as f:
        lines = f.readlines()

    line_1 = None
    line_E = None
    line_H = None

    for line in lines:
        if len(line.strip()) == 0:
            break
        if line[79].strip() == "1":
            line_1 = line
        elif line[79].strip() == "H":
            line_H = line
        elif line[79].strip() == "E":
            line_E = line

    origin = None
    origin_hypo = None
    magnitudes = []
    if line_1:
        seisan_hdr = parse_y2k_line(line_1, formats['1'])
        origin = seisan_hdr_to_origin(seisan_hdr)
        magnitudes = seisan_hdr_to_magnitudes(seisan_hdr)
    if line_H:
        seisan_hypo = parse_y2k_line(line_H, formats['H'])
        origin_hypo = seisan_hdr_to_origin(seisan_hypo)
    if line_E:
        seisan_errs = parse_y2k_line(line_E, formats['E'])
        add_seisan_errors_to_origin(seisan_errs, origin)
        origin.quality.azimuthal_gap = seisan_errs['Gap']

    if origin_hypo:
        origin.time = origin_hypo.time
        origin.latitude = origin_hypo.latitude
        origin.longitude = origin_hypo.longitude
        origin.depth = origin_hypo.depth

    for magnitude in magnitudes:
        magnitude.azimuthal_gap = origin.quality.azimuthal_gap

    ntype = '4'
    arrivals = []
    amplitudes = []
    picks = []
    for line in lines:
        if len(line.strip()) == 0:
            break
        if not line[79] in ['4', ' ']:
            continue

        parsed_type4 = parse_y2k_line(line, formats[ntype])
        # MTH: type4 Phase = 1-4 chars
        #    Probably it should look like:
        #12345678901234567890
        # ODSA HZ IP        65323.000                             125   -0.0510 7.71 108
        # ODSA H1 IS        65324.488                             125   -0.1710 7.71 108
        # ODSA H2 IAML      65324.691     15409.9 0.15                          7.71 108
        #    But it looks like (IAML col is right-shifted 1):
        #12345678901234567890
        # ODSA HZ IP        65323.000                             125   -0.0510 7.71 108
        # ODSA H1 IS        65324.488                             125   -0.1710 7.71 108
        # ODSA H2  IAML     65324.691     15409.9 0.15                          7.71 108
        #
        # valid pick phases: 'P', 'Pg', 'S', 'Sn'

        # Seisan highest weight = 0 or blank
        # If blank, parse_y2k_line will fill this in with default = -9
        if parsed_type4['Weighting_indicator'] < 0:
            parsed_type4['Weighting_indicator'] = 0

        if parsed_type4['Phase'][0] not in ['P', 'S', 'A', 'I']:
            logger.warning("nordic_to_qml: Phase=[%s] doesn't look like: 'P', 'S', or 'A' !" %
                           parsed_type4['Phase'])

        #parsed_type4['network_code'] = args.network_code if args and 'network_code' in args else 'UNK'
        if args.network_code is None:
            parsed_type4['network_code'] = 'UNK'
        else:
            parsed_type4['network_code'] = args.network_code


        if parsed_type4['Phase'][0] in ['A', 'I']:
            if args.fix_amptype:
                parsed_type4['amplitude_type'] = 'WAS'
                parsed_type4['amplitude_unit'] = 'other'

            amplitude = seisan_type4_to_amplitude(parsed_type4, origin)
            amplitudes.append(amplitude)
        else:
            arrival, pick = seisan_type4_to_arrival(parsed_type4, origin)
            arrivals.append(arrival)
            picks.append(pick)

    origin.arrivals = arrivals
    event = Event()
    event.origins = [origin]
    event.picks = picks
    event.preferred_origin_id = origin.resource_id.id
    event.magnitudes = magnitudes
    event.preferred_magnitude_id = magnitudes[0].resource_id.id
    event.amplitudes = amplitudes

    cat = Catalog(events=[event])

    return cat

def seisan_type4_to_amplitude(type4, origin):
    '''
    Convert a seisan type4 phase dict to obspy amplitude

    :param type4: parsed type4 line
    :type type4: python dict

    :return: converted phase 
    :rtype: obspy amplitude
    '''

    net=type4['network_code']
    sta=type4['sta']

    waveform_id = WaveformStreamID(net, sta)
    channel_code = "%sH%s" % (type4['Instrument_type'], type4['Component'])
    waveform_id.channel_code = channel_code
    # MTH:
    #waveform_id.location_code = "00"

    year = origin.time.year
    mo = origin.time.month
    dd = origin.time.day
    hh = type4['Hour']
    mi = type4['Minutes']
    ss = type4['Seconds']
    date = "%s-%s-%sT%s:%s:%s" % (year, mo, dd, hh, mi, ss)

    amplitude = Amplitude()
    amplitude.waveform_id = waveform_id
    amplitude.generic_amplitude = type4['Amplitude']
    amplitude.scaling_time = UTCDateTime(date)

    if 'amplitude_type' in type4:
        amplitude.type = type4['amplitude_type']
        amplitude.unit = type4['amplitude_unit']
    else:
        amplitude.type = 'AML' if 'AML' in type4['Phase'] else 'A'
        #amplitude.unit = ??

    if 'Period' in type4 and type4['Period'] > 0.:
        amplitude.period = type4['Period']

    return amplitude


def seisan_type4_to_arrival(type4, origin):
    '''
    Convert a seisan type4 phase dict to obspy arrival

    :param type4: parsed type4 line
    :type type4: python dict

    :return: converted phase 
    :rtype: obspy arrival and pick
    '''

    arrival = Arrival()
    phase = type4['Phase']
    arrival.phase = phase

    net=type4['network_code']
    sta=type4['sta']

    waveform_id = WaveformStreamID(net, sta)
    channel_code = "%sH%s" % (type4['Instrument_type'], type4['Component'])
    waveform_id.channel_code = channel_code
    # MTH:
    #waveform_id.location_code = "00"

    if type4['Azimuth_at_the_source'] != FIELD_UNSET:
        arrival.azimuth = type4['Azimuth_at_the_source']
    if type4['Azimuth_residual'] != FIELD_UNSET:
        arrival.backazimuth_residual = type4['Azimuth_residual']   # Could be wrong - is it even in the seisan manual ??
    if type4['Epicentral_distance'] != FIELD_UNSET:
        arrival.distance = type4['Epicentral_distance'] / 111.19   # obspy dist in deg
    if type4['Angle_of_incidence'] != FIELD_UNSET:
        arrival.takeoff_angle = type4['Angle_of_incidence']        # No idea if these are measured with same convention
    if type4['Travel_time_residual'] != FIELD_UNSET:
        arrival.time_residual = type4['Travel_time_residual']

    # Unmapped:              type4 field
    # somehow type4 I2      L   Weight ==> needs to map to arrival.time_weight (=float) ?
    #               I1      L   Weighting_indicator     (1-4) 0 or bank = full weight, 1=75%, 2=50%, 3=25%, 4=0%

    pick = Pick()
    arrival.pick_id = pick.resource_id
    pick.waveform_id = waveform_id
    year = origin.time.year
    mo = origin.time.month
    dd = origin.time.day
    hh = type4['Hour']
    mi = type4['Minutes']
    ss = type4['Seconds']
    date = "%s-%s-%sT%s:%s:%s" % (year, mo, dd, hh, mi, ss)
    pick.time = UTCDateTime(date)

    if type4['Phase_velocity'] != FIELD_UNSET:
        pick.horizontal_slowness = 111.19 / type4['Phase_velocity'] # km/s --> s/deg
    if type4['Direction_of_approach'] != FIELD_UNSET:
        pick.backazimuth = type4['Direction_of_approach']     # This is a guess - is it explained in the manual ??

    onset = type4['Quality_indicator']

    if onset == 'I':
        pick.onset = "impulsive"
    elif onset == 'E':
        pick.onset = "emergent"

    pick.phase_hint = phase

    if phase == 'P':
        if type4['First_motion'] == 'C':
            pick.polarity == 'positive'
        elif type4['First_motion'] == 'D':
            pick.polarity == 'negative'

    time_errors = QuantityError()
    #time_errors.uncertainty = hypoinv_weight_to_obspy_time_error(y2k_phase['%swtCode' % phase])
    #               I1      L   Weighting_indicator     (1-4) 0 or bank = full weight, 1=75%, 2=50%, 3=25%, 4=0%
    time_errors.uncertainty = hypoinv_weight_to_obspy_time_error(type4['Weighting_indicator'])
    pick.time_errors = time_errors

    return arrival, pick


def seisan_hdr_to_origin(seisan_hdr):
    '''
    Convert a seisan type1 or typeH line to quakeml Origin

    :param seisan_hdr: seisan type1/typeH parsed line
    :type seisan_hdr: python dict

    :return: converted origin 
    :rtype: obspy.core.event.origin.Origin
    '''

    origin = Origin()

    year = seisan_hdr['Year']

    mo = seisan_hdr['Month']
    dd = seisan_hdr['Day']
    hh = seisan_hdr['Hour']
    mi = seisan_hdr['Minutes']
    ss = seisan_hdr['Seconds']

    date = "%s-%s-%sT%s:%s:%s" % (year, mo, dd, hh, mi, ss)
    origin.time = UTCDateTime(date)

    origin.longitude = seisan_hdr['Longitude']
    origin.latitude = seisan_hdr['Latitude']
    origin.depth = seisan_hdr['Depth'] * 1.e3  # seisan depth [km] --> obspy depth [m]

    if 'Depth_indicator'in seisan_hdr:
        if seisan_hdr['Depth_indicator'] == 'F':
            origin.depth_type = "operator assigned"
    if 'Fix_o_time' in seisan_hdr:
        if seisan_hdr['Fix_o_time'] == 'F':
            origin.time_fixed = True

    quality = OriginQuality()
    if 'Number_of_stations_used' in seisan_hdr:
        quality.used_station_count = seisan_hdr['Number_of_stations_used']
    if 'RMS_of_time_residuals' in seisan_hdr:
        quality.standard_error  = seisan_hdr['RMS_of_time_residuals']

    origin.quality = quality

    creation_info = CreationInfo()
    if 'Hypocenter_reporting_agency' in seisan_hdr:
        creation_info.agency_id = seisan_hdr['Hypocenter_reporting_agency']
    origin.creation_info = creation_info

    return origin

def seisan_hdr_to_magnitudes(seisan_hdr):
    '''
    Convert a seisan type1 or typeH line to quakeml magnitudes

    :param seisan_hdr: seisan type1/typeH parsed line
    :type seisan_hdr: python dict

    :return: obspy magnitudes
    :rtype: list
    '''

    seisan_mag_types = {'L':'ML', 'b':'mb', 'B':'mB', 's':'Ms', 'S':'MS', 'W':'Mw', 'G':'MbLg', 'C':'Mc'}

    creation_info = None
    if seisan_hdr['Magnitude_reporting_agency'] != FIELD_UNSET and \
       len(seisan_hdr['Magnitude_reporting_agency'].strip()) > 0:
        creation_info = CreationInfo()
        creation_info.agency_id = seisan_hdr['Magnitude_reporting_agency']

    magnitudes = []
    for i in range(2):
        field = 'Magnitude_no_%d' % (i+1)
        if seisan_hdr[field] > FIELD_UNSET:
            #print("field:%s has value:%f" % (field, seisan_hdr[field]))
            magnitude = Magnitude()
            magnitude.mag = seisan_hdr[field]
            if seisan_hdr['Type_of_magnitude'] == FIELD_UNSET:
                mag_type = 'M' # default = unspecified
            elif seisan_hdr['Type_of_magnitude'] in seisan_mag_types:
                mag_type = seisan_mag_types[seisan_hdr['Type_of_magnitude']]
            else: # Use as is
                mag_type = seisan_hdr['Type_of_magnitude']

            magnitude.magnitude_type = mag_type
            # This is probably wrong - likely better to count the # of type4 line amp measurements
            magnitude.station_count = seisan_hdr['Number_of_stations_used'] 
            if creation_info:
                magnitude.creation_info = creation_info

            magnitudes.append(magnitude)

    return magnitudes


def add_seisan_errors_to_origin(seisan_errs, origin):
    '''
    seisan_errs = parsed seisan typeE line
    '''

    longitude_errors = QuantityError()
    longitude_errors.uncertainty = seisan_errs['Longitude_error']
    origin.longitude_errors = longitude_errors

    latitude_errors = QuantityError()
    latitude_errors.uncertainty = seisan_errs['Latitude_error']
    origin.latitude_errors = latitude_errors

    depth_errors = QuantityError()
    depth_errors.uncertainty = seisan_errs['Depth_error']
    origin.depth_errors = depth_errors

    time_errors = QuantityError()
    time_errors.uncertainty = seisan_errs['Origin_time_error']
    origin.time_errors = time_errors

    ''' What to do with this ?
    uncertainty = OriginUncertainty()
    seisan_hdrs['Covariance_xy']
    seisan_hdrs['Covariance_xz']
    seisan_hdrs['Covariance_yz']
    uncertainty.
    origin.uncertainty = uncertainty
horizontal_uncertainty (float, optional)
Circular confidence region, given by single value of horizontal uncertainty. Unit: m
min_horizontal_uncertainty (float, optional)
Semi-minor axis of confidence ellipse. Unit: m
max_horizontal_uncertainty (float, optional)
Semi-major axis of confidence ellipse. Unit: m
azimuth_max_horizontal_uncertainty (float, optional)
Azimuth of major axis of confidence ellipse. Measured clockwise from South-North direction at epicenter. Unit: deg
confidence_ellipsoid (ConfidenceEllipsoid, optional)
Confidence ellipsoid
preferred_description (str, optional)
Preferred uncertainty description. See OriginUncertaintyDescription for allowed values.
confidence_level (float, optional)
Confidence level of the uncertainty, given in percent.
    '''

    return



import os

# Quiet down matplotlib msgs (obspy must import matplotlib with logLevel=DEBUG)
#import logging
#logger = logging.getLogger('matplotlib')
#logger.setLevel(logging.WARNING)

from obspy.core.event import Catalog
from obspy.core.event import read_events
from obspy.core.event.event import Event
from obspy.core.event.magnitude import Magnitude

from .lib.liblog import getLogger, processCmdLine
from .lib.y2k_to_qml import y2k_to_quakeml

def main():

    logger = getLogger()
    arc_file, args = processCmdLine('y2kfile')

    infile = os.path.basename(arc_file)
    outfile = infile.replace('.y2k','.qml')
    if outfile == infile:
        logger.info("Infile doesn't have expected .y2k suffix --> Add .qml to infile")
        outfile += ".qml"

    logger.info("infile=%s --> outfile=./%s" % (arc_file, outfile))

    cat = y2k_to_quakeml(arc_file)
    cat.write(outfile, format="QUAKEML")

    return

if __name__ == "__main__":
    main()



# -*- coding: utf-8 -*-
from __future__ import print_function
#from ._version import get_versions

__author__ = 'Mike Hagerty'
__email__ = 'mhagerty@isti.com'
__version__ = '0.0.2'
#__version__ = get_versions()['version']
#del get_versions

import os
def get_format_dir():
    this_dir = os.path.abspath(os.path.dirname(__file__))
    formats_dir = os.path.join(this_dir, "formats")
    return formats_dir





import sys
import os

import collections
import math

from .liblog import getLogger

logger = getLogger()


def x_to_a(field_len, x):

    if abs(x) >= math.pow(10., field_len - 1):
        ndec = 0
    elif abs(x) >= 1:
        ndec = field_len - int(math.log10(abs(x))) - 2
        if ndec == 0:
            ndec = 1
    else:
        ndec = field_len - 1

    #if x < 0:
        #ndec -= 1

    xx = "%.*f" % (ndec, x)
    a  = str(xx)

    if x < 1:
        a = a.replace('0', '', 1)
    if len(a) > field_len:
        a = a[0:field_len]

    if len(a) != field_len:
        raise Exception
    else:
        return a

def new_y2k_line(y2kformat):
    d=collections.OrderedDict()
    #for key, value in y2kformat.iteritems():
    for key, value in y2kformat.items():
        d[key] = None
        #d[key] = printf("%*s" % (value['len'], ' '))
    return d

def write_y2000_phase(dformat, arrival):

    #print '1234567890123456789012345678901234567890123456789012345678901234567890'

    buf = ""

    for k, val in dformat.items():

        format = dformat[k]['format']
        w  = dformat[k]['len']

        # print("k=%s val=%s format=%s width=%d" % (k, val, dformat[k]['format'], w))

        if k[0:5] == 'blank' or arrival[k] is None or arrival[k] == '' or arrival[k] == -9:
            buf += ("%-*s" % (w, ' '))
        else:
            if format[0:1] == 'A':
                if val['align'] == 'R':
                    buf += ("%*s" % (w, str(arrival[k])))
                else:
                    buf += ("%-*s" % (w, str(arrival[k])))

            elif format[0:1] == 'I':
                if val['align'] == 'R':
                    buf += ("%*s" % (w, int(arrival[k])))
                else:
                    buf += ("%-*s" % (w, int(arrival[k])))

            elif format[0:1] == 'F':
                (field_len, ndec) = format[1:4].split('.')
                #print "format=%s field_len=%s ndec=%s" % (format, field_len, ndec)
                field_len = int(field_len)
                ndec = int(ndec)

                if k == 'Amp': 
                    buf += ("%*s" % (w, x_to_a(field_len, arrival[k])))
                else: 
                    data = float(arrival[k])*math.pow(10., ndec)
                    #print "k=[%s] arrival[k]=[%s] data=%.0f" % (k, arrival[k], data)
                    if val['align'] == 'R':
                        #printf("%*s" % (w, string))
                        a = "%*.0f" % (w, (data))
                        if len(a) > field_len:
                            #print "Houston we have a problem a=[%s]" % a
                            buf += ("%*s" % (w, x_to_a(field_len, arrival[k])))
                        else:
                            buf += ("%*.0f" % (w, (data)))
                    else:
                        #printf("%-*s" % (w, string))
                        a = "%-*.0f" % (w, (data))
                        if len(a) > field_len:
                            #print "Houston we have a problem a=[%s]" % a
                            buf += ("%*s" % (w, x_to_a(field_len, arrival[k])))
                        else:
                            buf += ("%-*.0f" % (w, (data)))
    buf += ("\n")
    return buf

def parse_y2k_line(line, hformat):
   dd = collections.OrderedDict()
   #for key, value in hformat.iteritems():
   #print("parse line=[%s]" % line)
   for key, value in hformat.items():
        i1 = value['start']-1
        i2 = i1 + value['len']
        data = line[i1:i2]
        form = value['format']
        #print("key=%s value=%s i1=%d i2=%d data=%s" % (key, value, i1, i2, data))

        if form[0] == 'A':
            dd[key] = data.strip()
	#MTH: hack to allow Prmk/Srmk to have leading blank - eg, Prmk=' P' or Prmk='iP', etc
            if key == 'Prmk' or key == 'Srmk':
                dd[key] = data
        elif form[0] == 'I':
            if not any(str.isdigit(c) for c in data):
                dd[key] = -9
                continue
            else:
                try:
                    data = int(data)
                    dd[key] = data
                except ValueError as e:
                    raise
        elif form[0] == 'F':
#MTH: What to do if field is empty in original y2000 file ?  
#     Flag with -9 and then don't print it ??
            if not any(str.isdigit(c) for c in data):
                dd[key] = -9
                #continue
            else:
                (foo, ndec) = form.split('.')
                try:
                    if '.' in data: # Try to read in float directly
                        data = float(data)
                    else:           # Read in int and convert using format specifier
                        ndec = int(ndec)
                        data = float(data)/math.pow(10., ndec)
                    dd[key] = data
                except Exception as e:
                    raise

        #endif form[0] == 'F'
        dd['string'] = line
   return dd


def read_format(file):
    d=collections.OrderedDict()
    f=open(file, 'r')
    #Skip first 4 lines
    lines=f.readlines()[4:]
    nblank=1
    for line in lines:
        line = line.rstrip('\n\r')
        parsedLine = line.split()
        if len(parsedLine) < 5:
            print("Error with line=[%s]" % line)
        else:
            try:
                col_start = int(parsedLine[0])
                col_len   = int(parsedLine[1])
                col_format= parsedLine[2]
                col_align = parsedLine[3]
                col_field = parsedLine[4]
                if col_field == 'blank':
                    col_field = 'blank%d' % nblank
                    nblank += 1
                if col_field == 'Free':
                    col_field = 'Free%d' % nblank
                    nblank += 1
                    #continue
                #else:
                dd = {}
                dd['start'] = col_start
                dd['len']   = col_len
                dd['format']= col_format
                dd['align'] = col_align
                d[col_field] = dd
            except:
                print("Exception!")
                print("parsedLine=[%s]" % parsedLine)

    f.close()
    return d


def printf(format, *args):
    sys.stdout.write(format % args)


def read_arc_shadow_file(file, dformat, hformat):
    fname = 'read_arc_shadow_file'
    d={}
    phase_list=[]
    #logger.info("Enter read_arc_shadow_file")
    try:
        f=open(file, 'r')
        # Read in origin/header line:
        origin = f.readline().rstrip('\n\r')
        origin_dict = parse_y2k_line(origin, hformat)
        origin_dict['string'] = origin
        lines=f.readlines()
    except (OSError, IOError) as e:
        logger.error("%s.%s: Attempt to read phase file=[%s] gives error=[%s]" % (__name__, fname, file, e))
        return None, None
    finally:
        f.close()

    if len(lines) == 0:
        logger.error("%s.%s: phase file=[%s] appears to be empty!" % (__name__, fname, file))
        exit(2)

    current_sta=''

    for line in lines:
        line = line.rstrip('\n\r')
        if not line:
            continue
        if line[0] == '$': #Read shadow line
            if len(line) < 2: #Empty shadow line
                continue
            elif len(line) == 34: #Current length of non-empty shadow lines - Does this vary ?
                (yyyymmddhhmi, alpha, logA0, beg, end) = read_coda_shadow(line[1:])
                coda_dict = {}
                coda_dict['yyyymmddhhmi'] = yyyymmddhhmi
                coda_dict['alpha'] = alpha
                coda_dict['logA0'] = logA0
                coda_dict['beg'] = beg
                coda_dict['end'] = end
                dd['coda'] = coda_dict
                continue
            else:
                logger.error("%s.%s: Expecting 34-char shadow line but len=[%d] line=[%s]" % \
                            (__name__, fname, len(line), line))

        else:
            dd = parse_y2k_line(line, dformat)
            current_sta = dd['sta']

        d[(dd['sta'], dd['chan'])] = dd
        phase_list.append(dd)
        #d[dd['sta']] = dd

    # Rekey from sta --> chan --> params & sort
    temp   = collections.OrderedDict(sorted(d.items()))
    phases = collections.OrderedDict()

    for (sta,chan) in temp:
        if not sta in phases:
            phases[sta] = collections.OrderedDict()
        phases[sta][chan] = temp[(sta,chan)]

    #return phases, origin
    #logger.info("DONE read_arc_shadow_file")
    # MTH: 2020-08-26 phases = dict[sta][cha] of arrivals while phase_list = [arrival] ordered as in arcfile
    #return phases, origin_dict
    return phase_list, origin_dict


def read_coda_shadow(buf):
    '''
Use the following format for the shadow line, which is similar to the format in which these data are stored in the modified UW1-format files.

Columns Format  Description
3-14     I4, 4I2  Reference date-time in the form YYYYMMDDHHMM where YYYY, MM, DD, HH, MM are 
                    the UTC year, month, day, hour, and minute
16-20    F5.2     Alpha (called Qfree by Jiggle), which is the decay constant in the coda 
                    decay function A0(t-tp)-alpha, where t is time and tp is the P-wave arrival time
22-26    F5.2     log(A0) (called Afree by Jiggle), where A0 is the amplitude constant in 
                    the coda decay function
28-30    I3       Begin time for coda-decay analysis in seconds relative to the reference time
32-34    I3       End time for coda-decay analysis in seconds relative to the reference time

12345678901234
$ 201112131506  1.50  4.31  75  94
    '''
    fname = 'read_coda_shadow'

    fields = buf.split()
    if len(fields) != 5:
        logger.error("%s: len(fields) != 5" % fname)
    else:
        (yyyymmddhhmi, alpha, logA0, beg, end) = fields
        try:
            yyyymmddhhmi = int(yyyymmddhhmi)
            alpha = float(alpha)
            logA0 = float(logA0)
            beg = int(beg)
            end = int(end)
        except (ValueError) as e:
            logger.error("%s.%s: Attempt to read shadow line=[%s] gives error=[%s]" % (__name__, fname, buf, e))
            raise

    return (yyyymmddhhmi, alpha, logA0, beg, end)

def write_coda_shadow(val, yyyymmddhhmi):
    '''
Use the following format for the shadow line, which is similar to the format in which these data are stored in the modified UW1-format files.

Columns Format  Description
3-14     I4, 4I2  Reference date-time in the form YYYYMMDDHHMM where YYYY, MM, DD, HH, MM are 
                    the UTC year, month, day, hour, and minute
16-20    F5.2     Alpha (called Qfree by Jiggle), which is the decay constant in the coda 
                    decay function A0(t-tp)-alpha, where t is time and tp is the P-wave arrival time
22-26    F5.2     log(A0) (called Afree by Jiggle), where A0 is the amplitude constant in 
                    the coda decay function
28-30    I3       Begin time for coda-decay analysis in seconds relative to the reference time
32-34    I3       End time for coda-decay analysis in seconds relative to the reference time

12345678901234
$ 201112131506  1.50  4.31  75  94
    '''

    printf("$ %12s %5.2f %5.2f %3d %3d\n" % (yyyymmddhhmi, val['alpha'], val['logA0'], val['beg'], val['end'])) 

    return





"""A setuptools based setup module for seismic-format"""
#!/usr/bin/env python
# -*- coding: utf-8 -*-

from codecs import open
from os import path
from setuptools import setup, find_packages

import seismic_format
#import versioneer

here = path.abspath(path.dirname(__file__))

with open(path.join(here, 'README.rst'), encoding='utf-8') as readme_file:
    readme = readme_file.read()

with open(path.join(here, 'HISTORY.rst'), encoding='utf-8') as history_file:
    history = history_file.read().replace('.. :changelog:', '')

requirements = [
    # TODO: put package requirements here
    'click',
    'obspy',
]

test_requirements = [
    # TODO: put package test requirements here
        'tox',
        'pytest',
        'pytest-runner',
]

cli_requirements = [
        'click',
]
develop_requirements = test_requirements + cli_requirements


setup(
    name='seismic-format',
    #version=versioneer.get_version(),
    version=seismic_format.__version__,
    #cmdclass=versioneer.get_cmdclass(),
    description="Collection of tools for converting between different seismic formats",
    #long_description=readme + '\n\n' + history,
    long_description=open('README.rst', 'r').read(),
    long_description_content_type='text/x-rst',
    author="Mike Hagerty",
    author_email='mhagerty@isti.com',
    url='https://gitlab.isti.com/mhagerty/seismic-format',
    packages=find_packages(exclude=['contrib', 'docs', 'tests']),
    entry_points={
        'console_scripts':[
            #'seismic-format=seismic_format.cli:cli',
            'quakeml-to-y2k=seismic_format.quakeml_to_y2k:main',
            'y2k-to-quakeml=seismic_format.y2k_to_quakeml:main',
            'seisan-to-quakeml=seismic_format.seisan_to_quakeml:main',
            ],
        },
    include_package_data=True,
    package_data={
        'seismic_format':['formats/format*']
    },
    #data_files=[('my_data', ['test_data/*'])],
    install_requires=requirements,
    license="MIT",
    classifiers=[
        'Development Status :: 2 - Pre-Alpha',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: MIT License',
        'Natural Language :: English',
        "Programming Language :: Python :: 2",
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.3',
        'Programming Language :: Python :: 3.4',
        'Programming Language :: Python :: 3.5',
    ],
    test_suite='tests',
    tests_require=test_requirements,
    extras_require={ # MTH: eg, >pip install pkg[cli]
        'cli': cli_requirements,
        'develop': develop_requirements,
        }
)



#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""
test_seismic_format
----------------------------------

Tests for `seismic_format` module.
"""

import filecmp
import glob
import os
import shutil
import sys
import unittest

import seismic_format
from seismic_format import get_format_dir
from seismic_format.lib.lib_y2k import read_arc_shadow_file
from seismic_format.lib.qml_to_y2k import quakeml_to_y2k
from seismic_format.lib.y2k_to_qml import y2k_to_quakeml, read_format
from seismic_format.lib.liblog import getLogger, processCmdLine
from seismic_format.lib.nordic_to_qml import seisan_to_quakeml

import logging
logger = getLogger()
logger.setLevel(logging.ERROR)

class TestSeismic_format(unittest.TestCase):

    @classmethod
    def setUpClass(cls):
        """
            class setup gets called once
        """
        formats_dir = get_format_dir()
        y2000_format_file = os.path.join(formats_dir, 'format.Y2000_station_archive')
        y2000_header_file = os.path.join(formats_dir, 'format.Y2000_header_archive')
        cls.y2k_format = read_format(y2000_format_file)
        cls.hdr_format = read_format(y2000_header_file)

    def setUp(self):
        """
            self setup gets called for each test
        """
        #formats_dir = get_format_dir()
        #y2000_format_file = os.path.join(formats_dir, 'format.Y2000_station_archive')
        #y2000_header_file = os.path.join(formats_dir, 'format.Y2000_header_archive')
        #self.assertEqual(os.path.isfile(y2000_header_file), 1)
        #self.y2k_format = read_format(y2000_format_file)
        #self.hdr_format = read_format(y2000_header_file)
        self.y2k_format = TestSeismic_format.y2k_format
        self.hdr_format = TestSeismic_format.hdr_format

    def test_read_arc(self):
        (y2k, y2k_origin)  = read_arc_shadow_file('test_data/70014958.y2k', self.y2k_format, self.hdr_format)
        self.assertEqual(y2k_origin['year'], 2020)
        self.assertEqual(y2k_origin['moddhhmi'], '03241839')
        self.assertEqual(y2k_origin['seconds'], 1.06)
        self.assertEqual(y2k_origin['lat_deg'], 35.0)
        self.assertEqual(y2k_origin['ampmag'], -0.64)
        self.assertEqual(y2k_origin['azgap'], 78)

    def test_read_arc_shadow(self):
        (y2k, y2k_origin)  = read_arc_shadow_file('test_data/uu_11121315063_arc2000_AQMS_export', self.y2k_format, self.hdr_format)
        # This also works:
        #assert y2k_origin['year'] == 2011
        self.assertEqual(y2k_origin['year'], 2011)
        self.assertEqual(y2k_origin['moddhhmi'], '12131506')
        self.assertEqual(y2k_origin['min_dist'], 4.0)
        self.assertEqual(y2k_origin['durmag'], 2.04)
        self.assertEqual(y2k_origin['pri_amp_mag_type'], 'L')
        self.assertEqual(y2k_origin['prefmag_label'], 'L')
        self.assertEqual(y2k_origin['prefmag'], 1.78)
        self.assertEqual(y2k_origin['alt_dur_mag'], 1.95)

        #self.assertEqual(chan_code[0], band_char, 'Channel code and band code are not equal.')

    def test_quakeml_to_y2k(self):
        infile  = 'test_data/70014958.qml'
        outfile = '70014958.y2k'
        y2k = quakeml_to_y2k(infile)
        with open(outfile, 'w') as fh:
            fh.write(y2k)
        #assert filecmp.cmp('test_data/70014958.y2k', outfile)
        self.assertEqual(filecmp.cmp('test_data/70014958.y2k', outfile), 1)


    def test_y2k_to_quakeml(self):
        infile  = 'test_data/70014958.y2k'
        outfile = '70014958.qml'
        cat = y2k_to_quakeml(infile)
        cat.write(outfile, format="QUAKEML")

        for arr in cat.events[0].preferred_origin().arrivals:
            pk = arr.pick_id.get_referred_object()
            h = pk.waveform_id
            if h.station_code == 'KB04' and h.channel_code == 'HHZ':
                self.assertEqual(pk.time_errors.uncertainty, 0.3)

    def test_seisan_to_quakeml(self):
        seisanfile = 'test_data/22-0106-00L.S202004'
        seisanfile = 'test_data/30-0653-00L.S201904.paul'
        qmlfile = '%s.qml' % os.path.basename(seisanfile)

        sys.argv = ['seisan-to-quakeml', '--infile', seisanfile,
                    #'--network-code=OK', '--fix-amptype',
                    '--loglevel=WARNING',
                    ]
        infile, args = processCmdLine('infile')
        cat = seisan_to_quakeml(infile, args)
        #cat.write(qmlfile, format="QUAKEML")
        for arr in cat.events[0].preferred_origin().arrivals:
            pk = arr.pick_id.get_referred_object()
            self.assertGreater(pk.time_errors.uncertainty, .001)

    def test_something(self):
        assert(seismic_format.__version__)


    @classmethod
    def tearDownClass(cls):
        files = glob.glob('70014958.*')
        for f in files:
            #print("Now remove file:%s" % f)
            os.remove(f)

if __name__ == "__main__":
    unittest.main()
